package data_manage

import (
	"fmt"
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

type EdbClassify struct {
	ClassifyId      int       `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	HasData         int       `description:"是否含有指标数据"`
	CreateTime      time.Time `description:"创建时间"`
	ModifyTime      time.Time `description:"修改时间"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	UniqueCode      string    `description:"唯一编码"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
}

func AddEdbClassify(item *EdbClassify) (lastId int64, err error) {
	o := orm.NewOrm()
	o.Using("data")
	lastId, err = o.Insert(item)
	return
}

type AddEdbClassifyReq struct {
	ClassifyName string `description:"分类名称"`
	ParentId     int    `description:"父级id,第一级传0"`
	Level        int    `description:"层级,第一级传0，其余传上一级的层级"`
}

func GetEdbClassifyCount(classifyName string, parentId int) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := `SELECT COUNT(1) AS count FROM edb_classify WHERE parent_id=? AND classify_name=? `
	err = o.Raw(sql, parentId, classifyName).QueryRow(&count)
	return
}

type EditEdbClassifyReq struct {
	ClassifyName string `description:"分类名称"`
	ClassifyId   int    `description:"分类名称"`
}

func GetEdbClassifyById(classifyId int) (item *EdbClassify, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := `SELECT * FROM edb_classify WHERE classify_id=? `
	err = o.Raw(sql, classifyId).QueryRow(&item)
	return
}

func GetEdbClassifyCountById(classifyId int) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := `SELECT count(1) AS count FROM edb_classify WHERE classify_id=? `
	err = o.Raw(sql, classifyId).QueryRow(&count)
	return
}

func EditEdbClassify(classifyId int, classifyName string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := `UPDATE edb_classify SET classify_name=?,modify_time=NOW() WHERE classify_id=? `
	_, err = o.Raw(sql, classifyName, classifyId).Exec()
	return
}

type DeleteEdbClassifyReq struct {
	ClassifyId int `description:"分类id"`
	EdbInfoId  int `description:"指标id"`
}

func GetEdbInfoCountByClassifyId(classifyId int) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_info AS a
				WHERE a.classify_id IN(
				SELECT t.classify_id FROM 
				(
				SELECT rd.*
				FROM (SELECT * FROM edb_classify WHERE parent_id IS NOT NULL) rd,
					 (SELECT @pid := ?) pd 
				WHERE FIND_IN_SET(parent_id, @pid) > 0 
				  AND @pid := CONCAT(@pid, ',', classify_id) 
				UNION SELECT * FROM edb_classify WHERE classify_id = @pid
				)AS t
				) `
	err = o.Raw(sql, classifyId).QueryRow(&count)
	return
}

func DeleteEdbClassify(classifyId int) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` DELETE FROM edb_classify
				WHERE classify_id IN(
				SELECT t.classify_id FROM
				(
				SELECT rd.*
				FROM (SELECT * FROM edb_classify WHERE parent_id IS NOT NULL) rd,
				(SELECT @pid := ?) pd
				WHERE FIND_IN_SET(parent_id, @pid) > 0
				AND @pid := CONCAT(@pid, ',', classify_id)
				UNION SELECT * FROM edb_classify WHERE classify_id = @pid
				)AS t
				) `
	_, err = o.Raw(sql, classifyId).Exec()
	return
}

func GetEdbClassifyByParentId(parentId int) (items []*EdbClassifyItems, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT * FROM edb_classify WHERE parent_id=?  order by sort asc,classify_id asc`
	_, err = o.Raw(sql, parentId).QueryRows(&items)
	return
}

func GetEdbClassifyAll() (items []*EdbClassifyItems, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT * FROM edb_classify WHERE parent_id<>0  order by sort asc,classify_id asc`
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

type EdbClassifyItems struct {
	ClassifyId      int `description:"分类id"`
	EdbInfoId       int `description:"指标id"`
	ClassifyName    string
	ParentId        int
	Level           int    `description:"层级"`
	Sort            int    `description:"排序字段，越小越靠前，默认值：10"`
	UniqueCode      string `description:"唯一编码"`
	Source          int    `description:"来源id"`
	SourceName      string `description:"来源名称"`
	SysUserId       int    `description:"创建人id"`
	SysUserRealName string `description:"创建人姓名"`
	StartDate       string
	EdbCode         string
	EdbType         int `description:"指标类型:1:基础指标,2:计算指标"`
	Children        []*EdbClassifyItems
}

type EdbClassifyListResp struct {
	AllNodes []*EdbClassifyItems
}

type ClassifyDeleteCheckResp struct {
	DeleteStatus int    `description:"检测状态:0:默认值，如果为0，继续走其他校验，1:若目录关联指标不可删除,2:确认删除当前目录及包含的子目录吗,3:当前指标已用作画图，不可删除"`
	TipsMsg      string `description:"提示信息"`
}

type ClassifyDeleteCheckReq struct {
	ClassifyId int `description:"分类id"`
	EdbInfoId  int `description:"指标id"`
}

func GetClassifyCountByClassifyId(classifyId int) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM (
			SELECT rd.*
			FROM (SELECT * FROM edb_classify WHERE parent_id IS NOT NULL) rd,
				 (SELECT @pid := ?) pd 
			WHERE FIND_IN_SET(parent_id, @pid) > 0 
			  AND @pid := CONCAT(@pid, ',', classify_id) 
			UNION SELECT * FROM edb_classify WHERE classify_id = @pid
			)AS t
			WHERE t.classify_id<>? `
	err = o.Raw(sql, classifyId, classifyId).QueryRow(&count)
	return
}

type EdbClassifySimplify struct {
	ClassifyId   int    `description:"分类id"`
	ClassifyName string `description:"分类名称"`
	ParentId     int
}

//递归获取所有目录
func GetClassifyAllByClassifyId(classifyId int) (resultList []*EdbClassifySimplify, err error) {
	fmt.Println("185")
	o := orm.NewOrm()
	o.Using("data")
	for {
		sql := ` SELECT * FROM edb_classify AS a
			WHERE a.classify_id=? order by sort asc,classify_id asc`
		item := new(EdbClassifySimplify)
		err = o.Raw(sql, classifyId).QueryRow(&item)
		if err != nil {
			break
		}
		if item.ClassifyId > 0 {
			resultList = append(resultList, item)
			classifyId = item.ParentId
		} else {
			break
		}
	}
	return resultList, err
}

// MoveEdbClassifyReq 移动分类请求参数
type MoveEdbClassifyReq struct {
	ClassifyId       int `description:"分类id"`
	ParentClassifyId int `description:"父级分类id"`
	PrevClassifyId   int `description:"上一个兄弟节点分类id"`
	NextClassifyId   int `description:"下一个兄弟节点分类id"`
}

// GetFirstEdbClassifyByParentId 获取当前父级分类下，且排序数相同 的排序第一条的数据
func GetFirstEdbClassifyByParentId(parentId int) (item *EdbClassify, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT * FROM edb_classify WHERE parent_id=? order by sort asc,classify_id asc limit 1`
	err = o.Raw(sql, parentId).QueryRow(&item)
	return
}

// UpdateEdbClassifySortByParentId 根据父类id更新排序
func UpdateEdbClassifySortByParentId(parentId, classifyId, nowSort int, updateSort string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` update edb_classify set sort = ` + updateSort + ` WHERE parent_id=? and sort > ? `
	if classifyId > 0 {
		sql += ` or ( classify_id > ` + fmt.Sprint(classifyId) + ` and sort = ` + fmt.Sprint(nowSort) + `)`
	}
	_, err = o.Raw(sql, parentId, nowSort).Exec()
	return
}

// Update 更新分类基础信息
func (edbClassify *EdbClassify) Update(cols []string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	_, err = o.Update(edbClassify, cols...)
	return
}

// GetEdbClassifyMaxSort 获取分类下最大的排序数
func GetEdbClassifyMaxSort(parentId int) (sort int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := `SELECT Max(sort) AS sort FROM edb_classify WHERE parent_id=? `
	err = o.Raw(sql, parentId).QueryRow(&sort)
	return
}
